<?php
/**
Copyright Guillaume Bourgeois (2018)

contact@souke.fr

Ce logiciel est un programme informatique servant à aider les producteurs
à distribuer leur production en circuits courts.

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
 */

namespace domain\Document\Document;

use common\helpers\Price;
use domain\Order\Order\Order;
use domain\Order\ProductOrder\ProductOrderSolver;
use domain\PointSale\PointSale\PointSale;
use domain\_\AbstractService;
use domain\_\SolverInterface;

class DocumentSolver extends AbstractService implements SolverInterface
{
    protected ProductOrderSolver $productOrderSolver;

    public function loadDependencies(): void
    {
        $this->productOrderSolver = $this->loadService(ProductOrderSolver::class);
    }

    public function getAmount(DocumentInterface $document, string $type = Order::AMOUNT_TOTAL, bool $format = false)
    {
        return $this->_getAmountGeneric($document, $type, false, $format);
    }

    public function getAmountWithTax(DocumentInterface $document, string $type = Order::AMOUNT_TOTAL, bool $format = false)
    {
        return $this->_getAmountGeneric($document, $type, true, $format);
    }

    protected function _getAmountGeneric(DocumentInterface $document, string $type = Order::AMOUNT_TOTAL, bool $withTax = true, bool $format = false)
    {
        $amount = 0;
        $totalVat = 0;

        foreach($this->getProductsOrders($document) as $productOrderArray) {
            foreach($productOrderArray as $productOrder) {
                $priceLine = $this->productOrderSolver->getPriceByTypeTotal($productOrder, $type) * $productOrder->quantity;
                $amount += $priceLine;
                $totalVat += Price::getVat($priceLine, $productOrder->taxRate->value, $document->tax_calculation_method);
            }
        }

        if ($this->isTaxCalculationMethodRoundingOfTheSum($document)) {
            $totalVat = Price::round($totalVat);
        }

        if ($withTax) {
            $amount += $totalVat;
        }

        if ($format) {
            return Price::format($amount);
        } else {
            return $amount;
        }
    }

    public function getTotalVatArray(DocumentInterface $document, string $typeTotal): array
    {
        $totalVatArray = [];

        // Méthode de calcul via getProductOrders()
        foreach($this->getProductsOrders($document) as $productOrderArray) {
            foreach ($productOrderArray as $productOrder) {

                $idTaxRate = $productOrder->taxRate->id;
                if (!isset($totalVatArray[$idTaxRate])) {
                    $totalVatArray[$idTaxRate] = 0;
                }

                $totalVatArray[$idTaxRate] += Price::getVat(
                    $this->productOrderSolver->getPriceByTypeTotal($productOrder, $typeTotal) * $productOrder->quantity,
                    $productOrder->taxRate->value,
                    $document->tax_calculation_method
                );
            }
        }

        return $totalVatArray;
    }

    public function getPointSale(DocumentInterface $document): ?PointSale
    {
        if (isset($document->orders) && isset($document->orders[0])) {
            return $document->orders[0]->pointSale;
        }

        return null;
    }

    public function getDistribution(DocumentInterface $document)
    {
        if (isset($document->orders) && isset($document->orders[0])) {
            return $document->orders[0]->distribution;
        }

        return null;
    }

    public function isDocumentDeliveryNote(DocumentInterface $document): bool
    {
        return $this->getClass($document) == 'DeliveryNote';
    }

    public function isDocumentInvoice(DocumentInterface $document): bool
    {
        return $this->getClass($document) == 'Invoice';
    }

    public function isDocumentQuotation(DocumentInterface $document): bool
    {
        return $this->getClass($document) == 'Quotation';
    }

    public function getClass(DocumentInterface $document, bool $pathComplete = false): string
    {
        $classDocument = get_class($document);

        if(!$pathComplete) {
            return str_replace(
                [
                    'domain\Document\DeliveryNote\\',
                    'domain\Document\Invoice\\',
                    'domain\Document\Quotation\\'],
                '',
                $classDocument);
        }

        return $classDocument;
    }

    public function getType(DocumentInterface $document): string
    {
        $class = $this->getClass($document);

        if ($class == 'Invoice') {
            $documentType = 'Facture';
        } elseif ($class == 'DeliveryNote') {
            $documentType = 'Bon de livraison';
        } elseif ($class == 'Quotation') {
            $documentType = 'Devis';
        }

        if (isset($documentType)) {
            return $documentType;
        }

        return '';
    }

    public function isValidClass(string $typeDocument): bool
    {
        return in_array($typeDocument, ['Invoice', 'DeliveryNote', 'Quotation']);
    }

    public function getStatusWording(DocumentInterface $document): string
    {
        return ($document->status == Document::STATUS_DRAFT) ? 'Brouillon' : 'Validé';
    }

    public function getStatusCssClass(DocumentInterface $document): string
    {
        return ($document->status == Document::STATUS_DRAFT) ? 'default' : 'success';
    }

    public function getHtmlLabel(DocumentInterface $document): string
    {
        $label = $this->getStatusWording($document);
        $classLabel = $this->getStatusCssClass($document);

        return '<span class="label label-' . $classLabel . '">' . $label . '</span>';
    }

    public function isStatus(DocumentInterface $document, string $status): bool
    {
        return $document->status == $status;
    }

    public function isStatusDraft(DocumentInterface $document): bool
    {
        return $this->isStatus($document, Document::STATUS_DRAFT);
    }

    public function isStatusValid(DocumentInterface $document): bool
    {
        return $this->isStatus($document, Document::STATUS_VALID);
    }

    public function getProductsOrders(DocumentInterface $document, bool $groupByInvoicePrice = false): array
    {
        $productsOrdersArray = [];
        $ordersArray = $document->orders;
        if ($ordersArray && count($ordersArray)) {
            foreach ($ordersArray as $order) {
                foreach ($order->productOrder as $productOrder) {
                    if ($productOrder->product) {
                        $indexProductOrder = $productOrder->product->order;
                        $newProductOrder = clone $productOrder;

                        if (!isset($productsOrdersArray[$indexProductOrder])) {
                            $productsOrdersArray[$indexProductOrder] = [$newProductOrder];
                        } else {
                            $productOrderMatch = false;
                            foreach ($productsOrdersArray[$indexProductOrder] as &$theProductOrder) {
                                if ($theProductOrder->unit == $productOrder->unit) {
                                    if($groupByInvoicePrice && $theProductOrder->invoice_price == $productOrder->invoice_price) {
                                        $theProductOrder->quantity += $productOrder->quantity;
                                        $productOrderMatch = true;
                                    }
                                    elseif($theProductOrder->price == $productOrder->price
                                        && $theProductOrder->invoice_price == $productOrder->invoice_price) {
                                        $theProductOrder->quantity += $productOrder->quantity;
                                        $productOrderMatch = true;
                                    }
                                }
                            }
                            if (!$productOrderMatch) {
                                $productsOrdersArray[$indexProductOrder][] = $newProductOrder;
                            }
                        }
                    }
                }
            }
        }

        // tri des orderProduct par product.order
        ksort($productsOrdersArray);

        return $productsOrdersArray;
    }

    public function getAliasDirectoryBase(DocumentInterface $document): string
    {
        return '@app/web/pdf/' . $document->id_producer . '/';
    }

    public function getFilename(DocumentInterface $document): string
    {
        $filename = $this->getType($document) . '-';

        if($this->isStatusValid($document)) {
            $filename .= $document->reference;
        }
        elseif($this->isStatusDraft($document)) {
            $filename .= 'Brouillon-'.$document->id;
        }

        $filename .= '.pdf';

        return $filename;
    }

    public function getFilenameComplete(DocumentInterface $document): string
    {
        return \Yii::getAlias($this->getAliasDirectoryBase($document) . $this->getFilename($document));
    }

    public function isInvoicePrice(DocumentInterface $document): bool
    {
        return $this->getClass($document) == 'Invoice' || $this->getClass($document) == 'DeliveryNote';
    }

    public function isTaxCalculationMethodSumOfRoundings(DocumentInterface $document): bool
    {
        return $document->tax_calculation_method == Document::TAX_CALCULATION_METHOD_SUM_OF_ROUNDINGS;
    }

    public function isTaxCalculationMethodRoundingOfTheSum(DocumentInterface $document): bool
    {
        return $document->tax_calculation_method == Document::TAX_CALCULATION_METHOD_ROUNDING_OF_THE_SUM;
    }
}